const express = require("express");
const router = express.Router();
const GetIssuer = require("./../../services/assetCodeGetterFromDB");
const DecryptWithServerKey = require("./../../services/decryptionservice");
const StellarSdk = require("stellar-sdk");
const CreateTrustline = require("./../../services/trustlineCreator");
const NFTRequest = require("./../../model/stellar/nftRequests");
const AssetCodeWithPractitioner = require("./../../model/stellar/assetCodeWithPractitioner");
const GenerateRandomAssetCode = require("./../../services/assetCodeGenerator");
require("dotenv").config();

router.post("/", async (req, res) => {
  const practitionerPK = DecryptWithServerKey(req.body.practitionerkey);
  const practitionerSK = DecryptWithServerKey(req.body.practitionerseed);
  const patientKey = DecryptWithServerKey(req.body.patient);
  const practitionerRSA = DecryptWithServerKey(req.body.practitionerrsa);

  //take the issuer key for the patient from the data base
  const issuerRes = await GetIssuer("patients", patientKey);

  //Generate the asset code
  GenerateRandomAssetCode()
    .then((assetCode) => {
      console.log("AC", assetCode);
      console.log("IP", DecryptWithServerKey(issuerRes.issuerKey));
      const issuerPK = DecryptWithServerKey(issuerRes.issuerKey);

      CreateTrustline(assetCode, issuerPK, practitionerSK)
        .then(async (result) => {
          console.log("Trust line created successfully!");
          //add request to DB
          const nftRequestObj = new NFTRequest({
            assetcode: assetCode,
            issuerpk: issuerPK,
            requesterpk: practitionerPK,
            requesterrsa: practitionerRSA,
            distributorpk: patientKey,
            status: "Pending",
          });
          try {
            const saveRequest = await nftRequestObj.save();

            //save the asset code details in the DB
            const assetCodeVsPractitioner = new AssetCodeWithPractitioner({
              assetcode: assetCode,
              to: practitionerPK,
            });

            try {
              const saveAssetCodeVsPrac = await assetCodeVsPractitioner.save();
              console.log("NFT request saved to database : " + saveRequest);
              res.status(200).json({
                message: "NFT request saved to database",
              });
              console.log("/practitioner/requestnft - requesting NFT success");
            } catch (errorWhenSavingPractitionerVsAssetCodeInDB) {
              console.log(
                "Error when adding asset code practitioner request to DB : ",
                errorWhenSavingPractitionerVsAssetCodeInDB
              );
              console.log("/practitioner/requestnft - requesting NFT failed");
              return res.status(500).json({
                message: "Error when adding NFT request to DB : ",
                errorWhenSavingPractitionerVsAssetCodeInDB,
              });
            }
          } catch (errorWhenAddingRequestToDB) {
            console.log(
              "Error when adding NFT request to DB : ",
              errorWhenAddingRequestToDB
            );
            console.log("/practitioner/requestnft - requesting NFT failed");
            return res.status(500).json({
              message: "Error when adding NFT request to DB : ",
              errorWhenAddingRequestToDB,
            });
          }
        })
        .catch((error) => {
          console.log("Failed to create the trust line  : ", error);
          console.log("/practitioner/requestnft - requesting NFT failed");
          return res.status(500).json({
            message: "Failed to create the trust line  : ",
            error,
          });
        });
    })
    .catch((errWhenGeneratingAssetCode) => {
      console.log(
        "Failed to generate asset code  : ",
        errWhenGeneratingAssetCode
      );
      console.log("/practitioner/requestnft - requesting NFT failed");
      return res.status(500).json({
        message: "Failed to generate asset code  : ",
        errWhenGeneratingAssetCode,
      });
    });
});

module.exports = router;
