import { useEffect, useState } from "react";
import { LOCALBACKEND } from "../../env";
import { Button, StyleSheet, View, Text, Alert } from "react-native";
import AsyncStorage from "@react-native-async-storage/async-storage";
import EncryptWithServerKey from "../../services/encryptByServerKey";
import Toast from "react-native-toast-message";

export default function PatientNFTRequests({ navigation }) {
  const [data, setData] = useState([]);
  const [assetData, setAssetData] = useState();

  useEffect(() => {
    fetchRequests();
  }, []);

  const fetchRequests = async () => {
    //get key from async storage
    try {
      const patientKey = await AsyncStorage.getItem("patientStellarPublicKey");
      const patientSeed = await AsyncStorage.getItem("patientStellarSecretKey");
      try {
        const requestUrl = LOCALBACKEND + `/nftrequests/${patientKey}`;

        console.log(requestUrl);

        const response = await fetch(requestUrl);
        const json = await response.json();
        setData(json.data);
      } catch (errorFetchingRequests) {
        console.log(
          "Error when fetching request data : ",
          errorFetchingRequests
        );
      }
    } catch (errorWhenGettingKey) {
      console.log(
        "Error when getting the public key from the async storage : ",
        errorWhenGettingKey
      );
    }
  };

  const renderItem = (item) => {
    return (
      <View key={item._id}>
        <Text>Key: {item.requesterpk}</Text>
        <Button title="Send NFT" onPress={() => handleSendNFT(item)}></Button>
      </View>
    );
  };

  const handleSendNFT = async (item) => {
    //get the asset code for the specific practitioner
    try {
      const patientKey = await AsyncStorage.getItem("patientStellarPublicKey");
      const patientSeed = await AsyncStorage.getItem("patientStellarSecretKey");

      try {
        const assetUrl =
          LOCALBACKEND + `/practitioner/getassetcode/${item.requesterpk}`;

        const response = await fetch(assetUrl);
        const json = await response.json();
        setAssetData(json.assetcode);

        //send the NFT
        const practitionerPubKey = item.requesterpk;
        const practitionerRsaKey = item.requesterrsa;
        const assetCode = assetData;

        const sendNFTObj = {
          patientkey: patientKey,
          patientseed: patientSeed,
          practitioner: practitionerPubKey,
          practitionerrsa: practitionerRsaKey,
          assetcode: assetCode,
        };

        const nftUrl = LOCALBACKEND + `/patient/createnft`;
        try {
          const nftResponse = await fetch(nftUrl, {
            method: "POST",
            headers: {
              "Content-Type": "application/json",
            },
            body: JSON.stringify(sendNFTObj),
          });

          const nftResponseData = await nftResponse.json();

          Toast.show({
            type: "success", // 'success', 'error', 'info', 'warning'
            text1: "EHR sent",
            visibilityTime: 3000, // Duration to show the toast in milliseconds
            autoHide: true,
          });
        } catch (errorWhenSendingNft) {
          console.error("Error when sending NFT : ", errorWhenSendingNft);
        }
      } catch (errorWhenGettingAssetCode) {
        console.error(
          "Error when getting the asset code : ",
          errorWhenGettingAssetCode
        );
      }
    } catch (errorWhenGettingKeyPair) {
      console.error("Error when getting key pair : ", errorWhenGettingKeyPair);
    }
  };

  return (
    <View style={styles.container}>
      {data.map((item) => renderItem(item))}
      <Toast ref={(ref) => Toast.setRef(ref)} />
    </View>
  );
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    justifyContent: "center",
    alignItems: "center",
  },
  text: {
    fontSize: 24,
    fontWeight: "bold",
  },
});
